@@ -13,10 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import collections
+from importlib import resources
 import logging
 import os
 
-import pkg_resources
 import requests
 
 from sushy import auth as sushy_auth
@@ -41,7 +41,7 @@ from sushy import utils
 
 LOG = logging.getLogger(__name__)
 
-STANDARD_REGISTRY_PATH = 'standard_registries/'
+STANDARD_REGISTRY_PATH = 'standard_registries'
 
 
 class ProtocolFeaturesSupportedField(base.CompositeField):
@@ -158,7 +158,8 @@ class Sushy(base.ResourceBase):
                  root_prefix='/redfish/v1/', verify=True,
                  auth=None, connector=None,
                  public_connector=None,
-                 language='en'):
+                 language='en', server_side_retries=10,
+                 server_side_retries_delay=3):
         """A class representing a RootService
 
         :param base_url: The base URL to the Redfish controller. It
@@ -181,6 +182,10 @@ class Sushy(base.ResourceBase):
             on the Internet, e.g., for Message Registries. Defaults to None.
         :param language: RFC 5646 language code for Message Registries.
             Defaults to 'en'.
+        :param server_side_retries: Number of times to retry GET requests in
+            case of server side errors. Defaults to 10.
+        :param server_side_retries_delay: Time in seconds between retries of
+            GET requests in case of server side errors. Defaults to 3.
         """
         self._root_prefix = root_prefix
         if (auth is not None and (password is not None
@@ -194,7 +199,10 @@ class Sushy(base.ResourceBase):
         self._auth = auth
 
         super(Sushy, self).__init__(
-            connector or sushy_connector.Connector(base_url, verify=verify),
+            connector or sushy_connector.Connector(
+                base_url, verify=verify,
+                server_side_retries=server_side_retries,
+                server_side_retries_delay=server_side_retries_delay),
             path=self._root_prefix)
         self._public_connector = public_connector or requests
         self._language = language
@@ -387,7 +395,7 @@ class Sushy(base.ResourceBase):
             links_url = self.json.get('Links')
             sessions_uri = links_url['Sessions']['@odata.id']
             # Save the session URL for post detection and prevention
-            # of recursive autentication attempts.
+            # of recursive authentication attempts.
             self._conn._sessions_uri = sessions_uri
             return sessions_uri
         except (TypeError, KeyError):
@@ -422,7 +430,6 @@ class Sushy(base.ResourceBase):
         :raises: HTTPError
         :raises: MissingAttributeError
         """
-
         # Explicitly removes in-client session data to proceed. This prevents
         # AccessErrors as prior authentication shouldn't be submitted with a
         # new authentication attempt, and doing so with old/invalid session
@@ -457,6 +464,13 @@ class Sushy(base.ResourceBase):
                 method='POST', url=session_service_path, response=rsp)
 
         session_uri = rsp.headers.get('Location')
+        if session_uri is None:
+            try:
+                body = rsp.json()
+                session_uri = body.get("@odata.id")
+            except ValueError:  # JSON decoding failed
+                pass
+
         if session_uri is None:
             LOG.warning("Received X-Auth-Token but NO session uri.")
 
@@ -554,18 +568,17 @@ class Sushy(base.ResourceBase):
         :returns: list of MessageRegistry
         """
 
-        message_registries = []
-        resource_package_name = __name__
-        for json_file in pkg_resources.resource_listdir(
-                resource_package_name, STANDARD_REGISTRY_PATH):
-            # Not using path.join according to pkg_resources docs
-            mes_reg = message_registry.MessageRegistry(
-                None, STANDARD_REGISTRY_PATH + json_file,
-                reader=base.JsonPackagedFileReader(
-                    resource_package_name))
-            message_registries.append(mes_reg)
-
-        return message_registries
+        return [
+            message_registry.MessageRegistry(
+                None,
+                os.path.join(STANDARD_REGISTRY_PATH, json_file.name),
+                reader=base.JsonPackagedFileReader(__package__),
+            )
+            for json_file in resources.files(__package__)
+            .joinpath(STANDARD_REGISTRY_PATH)
+            .iterdir()
+            if json_file.is_file()
+        ]
 
     @property
     @utils.cache_it
@@ -590,6 +603,7 @@ class Sushy(base.ResourceBase):
 
         registry_col = self._get_registry_collection()
 
+        endpoint_registries = {}
         if registry_col:
             provided = registry_col.get_members()
             for r in provided:
@@ -603,9 +617,16 @@ class Sushy(base.ResourceBase):
                         self._language,
                         self._public_connector)
                 if registry:
-                    registries[r.registry] = registry
-                    if r.identity not in registries:
-                        registries[r.identity] = registry
+                    endpoint_registries[r.registry] = registry
+                    endpoint_registries.setdefault(r.identity, registry)
+
+        if endpoint_registries:
+            LOG.debug('Found registries for %(id)s: %(reg)s',
+                      {'id': self.identity,
+                       'reg': ', '.join(endpoint_registries)})
+            registries.update(endpoint_registries)
+        else:
+            LOG.debug('No registries are available for %s', self.identity)
 
         return registries
 
