@@ -23,6 +23,7 @@ from sushy import exceptions
 from sushy.resources.chassis import chassis
 from sushy.resources import constants as res_cons
 from sushy.resources.manager import manager
+from sushy.resources.manager import virtual_media
 from sushy.resources.oem import fake
 from sushy.resources.system import bios
 from sushy.resources.system import processor
@@ -82,6 +83,14 @@ class SystemTestCase(base.TestCase):
                          .maintenance_window_start_time)
         for oem_vendor in self.sys_inst.oem_vendors:
             self.assertIn(oem_vendor, ('Contoso', 'Chipwise'))
+        self.assertEqual(sushy.BootProgressStates.OS_RUNNING,
+                         self.sys_inst.boot_progress.last_state)
+        self.assertEqual(66,
+                         self.sys_inst.boot_progress.last_boot_seconds_count)
+        self.assertEqual(parser.parse('2017-05-03T23:12:37-05:00'),
+                         self.sys_inst.boot_progress.last_state_updated_at)
+        self.assertEqual("OS foo running.",
+                         self.sys_inst.boot_progress.oem_last_state)
 
     def test__parse_attributes_return(self):
         attributes = self.sys_inst._parse_attributes(self.json_doc)
@@ -495,6 +504,35 @@ class SystemTestCase(base.TestCase):
                            'HttpBootUri': 'http://test.lan/test_image.iso'}},
             etag=mock.ANY)
 
+    def test_set_system_boot_options_httpboot_settings(self):
+
+        with open('sushy/tests/unit/json_samples/settings-nokia.json') as f:
+            settings_obj = json.load(f)
+
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        with open('sushy/tests/unit/json_samples/'
+                  'settings-body-nokia.json') as f:
+            settings_body = json.load(f)
+
+        get_settings = mock.MagicMock(headers={'ETag': '"3d7b838291941d"'})
+        get_settings.json.return_value = settings_body
+        get_system = mock.MagicMock(headers={'ETag': '"222"'})
+        self.conn.get.side_effect = [get_settings, get_system]
+
+        self.sys_inst.set_system_boot_options(
+            sushy.BootSource.UEFI_HTTP,
+            enabled=sushy.BootSourceOverrideEnabled.ONCE,
+            http_boot_uri='http://test.lan/test_image.iso'
+            )
+        self.sys_inst._conn.patch.assert_called_once_with(
+            '/redfish/v1/Systems/Self/SD',
+            data={'Boot': {'BootSourceOverrideTarget': 'UefiHttp',
+                           'BootSourceOverrideEnabled': 'Once',
+                           'HttpBootUri': 'http://test.lan/test_image.iso'}},
+            etag=mock.ANY)
+
     def test_set_system_boot_options_httpboot(self):
         self.sys_inst.set_system_boot_options(
             sushy.BootSource.UEFI_HTTP,
@@ -881,6 +919,33 @@ class SystemTestCase(base.TestCase):
         self.assertEqual(
             '/redfish/v1/Managers/BMC', actual_managers[0].path)
 
+    def test_managers_retry(self):
+        with open('sushy/tests/unit/json_samples/'
+                  'system_managedby_missing_manager_present.json') as f:
+            settings_obj = json.load(f)
+        self.json_doc.update(settings_obj)
+        self.sys_inst._parse_attributes(self.json_doc)
+
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'manager.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        actual_managers = self.sys_inst.managers
+        self.assertIsInstance(actual_managers[0], manager.Manager)
+        self.assertEqual(
+            '/redfish/v1/Managers/BMC', actual_managers[0].path)
+
+    def test_managers_missing_ref(self):
+        # | GIVEN |
+        del self.json_doc['Links']['ManagedBy']
+
+        # | WHEN & THEN |
+        with self.assertRaisesRegex(
+            exceptions.MissingAttributeError, 'attribute Links/ManagedBy'):
+            self.sys_inst.managers
+
     def test_chassis(self):
         # | GIVEN |
         with open('sushy/tests/unit/json_samples/'
@@ -902,6 +967,85 @@ class SystemTestCase(base.TestCase):
         self.assertIs(self.sys_inst, contoso_system_extn_inst._parent_resource)
         self.assertEqual('Contoso', contoso_system_extn_inst._vendor_id)
 
+    def test_no_virtual_media_attr(self):
+        with self.assertRaisesRegex(
+            exceptions.MissingAttributeError, 'attribute VirtualMedia'):
+            self.sys_inst.virtual_media
+
+
+class SystemWithVirtualMedia(base.TestCase):
+
+    def setUp(self):
+        super(SystemWithVirtualMedia, self).setUp()
+        self.conn = mock.Mock()
+        with open('sushy/tests/unit/json_samples/'
+                  'systemv1_20.json') as f:
+            self.json_doc = json.load(f)
+
+        self.conn.get.return_value.json.return_value = self.json_doc
+
+        self.sys_inst = system.System(
+            self.conn, '/redfish/v1/Systems/437XR1138R2',
+            redfish_version='1.0.2')
+
+    def test_virtual_media(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_mediav1_6.json') as f:
+            virtual_media_json = json.load(f)
+
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_media_collectionv1_6.json') as f:
+            virtual_media_collection_json = json.load(f)
+
+        self.conn.get.return_value.json.side_effect = [
+            virtual_media_collection_json, virtual_media_json]
+
+        # | WHEN |
+        actual_virtual_media = self.sys_inst.virtual_media
+
+        # | THEN |
+        self.assertIsInstance(actual_virtual_media,
+                              virtual_media.VirtualMediaCollection)
+        self.assertEqual(actual_virtual_media.name, 'Virtual Media Services')
+
+        member = actual_virtual_media.get_member(
+            '/redfish/v1/Systems/437XR1138R2/VirtualMedia/CD1')
+
+        self.assertEqual(member.image_name, 'mymedia-read-only')
+        self.assertTrue(member.inserted)
+        self.assertFalse(member.write_protected)
+        self.assertTrue(member.verify_certificate)
+
+    def test_virtual_media_on_refresh(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_media_collectionv1_6.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        actual_virtual_media = self.sys_inst.virtual_media
+        self.assertIsInstance(actual_virtual_media,
+                              virtual_media.VirtualMediaCollection)
+
+        with open('sushy/tests/unit/json_samples/systemv1_20.json', 'r') as f:
+            self.conn.get.return_value.json.return_value = json.loads(f.read())
+
+        self.sys_inst.invalidate()
+        self.sys_inst.refresh(force=False)
+
+        self.assertTrue(actual_virtual_media._is_stale)
+
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'virtual_media_collectionv1_6.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.virtual_media,
+                              virtual_media.VirtualMediaCollection)
+        self.assertFalse(actual_virtual_media._is_stale)
+
 
 class SystemCollectionTestCase(base.TestCase):
 
