@@ -59,7 +59,7 @@ class ConnectorMethodsTestCase(base.TestCase):
                       headers=self.headers.copy())
         mock__op.assert_called_once_with(mock.ANY, 'GET', 'fake/path',
                                          data=self.data, headers=self.headers,
-                                         blocking=False, timeout=60)
+                                         blocking=False, timeout=None)
 
     @mock.patch.object(connector.Connector, '_op', autospec=True)
     def test_get_blocking(self, mock__op):
@@ -67,7 +67,7 @@ class ConnectorMethodsTestCase(base.TestCase):
                       headers=self.headers.copy(), blocking=True)
         mock__op.assert_called_once_with(mock.ANY, 'GET', 'fake/path',
                                          data=self.data, headers=self.headers,
-                                         blocking=True, timeout=60)
+                                         blocking=True, timeout=None)
 
     @mock.patch.object(connector.Connector, '_op', autospec=True)
     def test_post(self, mock__op):
@@ -75,7 +75,7 @@ class ConnectorMethodsTestCase(base.TestCase):
                        headers=self.headers.copy())
         mock__op.assert_called_once_with(mock.ANY, 'POST', 'fake/path',
                                          data=self.data, headers=self.headers,
-                                         blocking=False, timeout=60)
+                                         blocking=False, timeout=None)
 
     @mock.patch.object(connector.Connector, '_op', autospec=True)
     def test_post_blocking(self, mock__op):
@@ -91,15 +91,15 @@ class ConnectorMethodsTestCase(base.TestCase):
                         headers=self.headers.copy())
         mock__op.assert_called_once_with(mock.ANY, 'PATCH', 'fake/path',
                                          data=self.data, headers=self.headers,
-                                         blocking=False, timeout=60)
+                                         blocking=False, timeout=None)
 
     @mock.patch.object(connector.Connector, '_op', autospec=True)
     def test_patch_blocking(self, mock__op):
-        self.conn.patch(path='fake/path', data=self.data.copy(),
+        self.conn.patch(path='fake/path', data=self.data.copy(), timeout=120,
                         headers=self.headers.copy(), blocking=True)
         mock__op.assert_called_once_with(mock.ANY, 'PATCH', 'fake/path',
                                          data=self.data, headers=self.headers,
-                                         blocking=True, timeout=60)
+                                         blocking=True, timeout=120)
 
     @mock.patch.object(connector.Connector, '_op', autospec=True)
     def test_put(self, mock__op):
@@ -107,15 +107,15 @@ class ConnectorMethodsTestCase(base.TestCase):
                       headers=self.headers.copy())
         mock__op.assert_called_once_with(mock.ANY, 'PUT', 'fake/path',
                                          data=self.data, headers=self.headers,
-                                         blocking=False, timeout=60)
+                                         blocking=False, timeout=None)
 
     @mock.patch.object(connector.Connector, '_op', autospec=True)
     def test_put_blocking(self, mock__op):
-        self.conn.put(path='fake/path', data=self.data.copy(),
+        self.conn.put(path='fake/path', data=self.data.copy(), timeout=120,
                       headers=self.headers.copy(), blocking=True)
         mock__op.assert_called_once_with(mock.ANY, 'PUT', 'fake/path',
                                          data=self.data, headers=self.headers,
-                                         blocking=True, timeout=60)
+                                         blocking=True, timeout=120)
 
     @mock.patch.object(connector.Connector, '_op', autospec=True)
     def test_delete(self, mock__op):
@@ -123,15 +123,15 @@ class ConnectorMethodsTestCase(base.TestCase):
                          headers=self.headers.copy())
         mock__op.assert_called_once_with(mock.ANY, 'DELETE', 'fake/path',
                                          data=self.data, headers=self.headers,
-                                         blocking=False, timeout=60)
+                                         blocking=False, timeout=None)
 
     @mock.patch.object(connector.Connector, '_op', autospec=True)
     def test_delete_blocking(self, mock__op):
-        self.conn.delete(path='fake/path', data=self.data.copy(),
+        self.conn.delete(path='fake/path', data=self.data.copy(), timeout=120,
                          headers=self.headers.copy(), blocking=True)
         mock__op.assert_called_once_with(mock.ANY, 'DELETE', 'fake/path',
                                          data=self.data, headers=self.headers,
-                                         blocking=True, timeout=60)
+                                         blocking=True, timeout=120)
 
     def test_set_auth(self):
         mock_auth = mock.MagicMock()
@@ -168,17 +168,31 @@ class ConnectorOpTestCase(base.TestCase):
             server_side_retries=10, server_side_retries_delay=3)
         self.conn._auth = mock_auth
         self.data = {'fake': 'data'}
-        self.headers = {'X-Fake': 'header'}
+        self.headers = {'Accept-Encoding': 'identity', 'OData-Version': '4.0'}
         self.session = mock.Mock(spec=requests.Session)
         self.conn._session = self.session
         self.request = self.session.request
         self.request.return_value.status_code = http_client.OK
 
     def test_ok_get(self):
-        self.conn._op('GET', path='fake/path', headers=self.headers)
+        self.conn._op('GET', path='fake/path')
         self.request.assert_called_once_with(
             'GET', 'http://foo.bar:1234/fake/path',
-            headers=self.headers, json=None, verify=True)
+            headers=self.headers, json=None, verify=True, timeout=60)
+
+    def test_ok_get_with_headers(self):
+        self.conn._op('GET', path='fake/path', headers={'answer': '42'})
+        self.request.assert_called_once_with(
+            'GET', 'http://foo.bar:1234/fake/path',
+            headers=dict(self.headers, answer='42'),
+            json=None, verify=True, timeout=60)
+
+    def test_ok_get_default_timeout(self):
+        self.conn._default_request_timeout = 42
+        self.conn._op('GET', path='fake/path')
+        self.request.assert_called_once_with(
+            'GET', 'http://foo.bar:1234/fake/path',
+            headers=self.headers, json=None, verify=True, timeout=42)
 
     def test_response_callback(self):
         mock_response_callback = mock.MagicMock()
@@ -188,34 +202,42 @@ class ConnectorOpTestCase(base.TestCase):
         self.assertEqual(1, mock_response_callback.call_count)
 
     def test_ok_get_url_redirect_false(self):
-        self.conn._op('GET', path='fake/path', headers=self.headers,
-                      allow_redirects=False)
+        self.conn._op('GET', path='fake/path', allow_redirects=False)
         self.request.assert_called_once_with(
             'GET', 'http://foo.bar:1234/fake/path',
             headers=self.headers, json=None, allow_redirects=False,
-            verify=True)
+            verify=True, timeout=60)
 
     def test_ok_post(self):
-        self.conn._op('POST', path='fake/path', data=self.data.copy(),
-                      headers=self.headers)
+        self.conn._op('POST', path='fake/path', data=self.data.copy())
         self.request.assert_called_once_with(
             'POST', 'http://foo.bar:1234/fake/path',
-            json=self.data, headers=self.headers, verify=True)
+            json=self.data,
+            headers=dict(self.headers, **{'Content-Type': 'application/json'}),
+            verify=True, timeout=60)
+
+    def test_ok_post_with_headers(self):
+        self.conn._op('POST', path='fake/path', data=self.data.copy(),
+                      headers={'answer': 42})
+        self.request.assert_called_once_with(
+            'POST', 'http://foo.bar:1234/fake/path',
+            json=self.data,
+            headers=dict(self.headers, **{'Content-Type': 'application/json',
+                                          'answer': 42}),
+            verify=True, timeout=60)
 
     def test_ok_put(self):
-        self.conn._op('PUT', path='fake/path', data=self.data.copy(),
-                      headers=self.headers)
+        self.conn._op('PUT', path='fake/path', data=self.data.copy())
         self.request.assert_called_once_with(
             'PUT', 'http://foo.bar:1234/fake/path',
-            json=self.data, headers=self.headers, verify=True)
+            headers=dict(self.headers, **{'Content-Type': 'application/json'}),
+            json=self.data, verify=True, timeout=60)
 
     def test_ok_delete(self):
-        expected_headers = self.headers.copy()
-        expected_headers['OData-Version'] = '4.0'
-        self.conn._op('DELETE', path='fake/path', headers=self.headers.copy())
+        self.conn._op('DELETE', path='fake/path')
         self.request.assert_called_once_with(
             'DELETE', 'http://foo.bar:1234/fake/path',
-            headers=expected_headers, json=None, verify=True)
+            headers=self.headers, json=None, verify=True, timeout=60)
 
     def test_ok_post_with_session(self):
         self.conn._session.headers = {}
@@ -227,7 +249,7 @@ class ConnectorOpTestCase(base.TestCase):
                       data=self.data)
         self.request.assert_called_once_with(
             'POST', 'http://foo.bar:1234/fake/path',
-            json=self.data, headers=expected_headers, verify=True)
+            json=self.data, headers=expected_headers, verify=True, timeout=60)
         self.assertEqual(self.conn._session.headers,
                          {'X-Auth-Token': 'asdf1234'})
 
@@ -240,24 +262,25 @@ class ConnectorOpTestCase(base.TestCase):
         self.conn._op('GET', path=path, headers=headers)
         self.request.assert_called_once_with(
             'GET', 'http://foo.bar:1234' + path,
-            headers=expected_headers, json=None, verify=True)
-
-    def test_odata_version_header_redfish_no_headers(self):
-        path = '/redfish/v1/bar'
-        expected_headers = {'OData-Version': '4.0'}
-        self.conn._op('GET', path=path)
-        self.request.assert_called_once_with(
-            'GET', 'http://foo.bar:1234' + path,
-            headers=expected_headers, json=None, verify=True)
+            headers=expected_headers, json=None, verify=True, timeout=60)
 
     def test_odata_version_header_redfish_existing_header(self):
         path = '/redfish/v1/foo'
         headers = {'OData-Version': '3.0'}
-        expected_headers = dict(headers)
+        expected_headers = dict(self.headers, **headers)
         self.conn._op('GET', path=path, headers=headers)
         self.request.assert_called_once_with(
             'GET', 'http://foo.bar:1234' + path,
-            headers=expected_headers, json=None, verify=True)
+            headers=expected_headers, json=None, verify=True, timeout=60)
+
+    def test_remove_header_accept_encoding(self):
+        path = '/redfish/v1/foo'
+        headers = {'Accept-Encoding': None}
+        self.headers.pop('Accept-Encoding')
+        self.conn._op('GET', path=path, headers=headers)
+        self.request.assert_called_once_with(
+            'GET', 'http://foo.bar:1234' + path,
+            headers=self.headers, json=None, verify=True, timeout=60)
 
     def test_timed_out_session_unable_to_create_session(self):
         self.conn._auth.can_refresh_session.return_value = False
@@ -335,6 +358,46 @@ class ConnectorOpTestCase(base.TestCase):
         self.auth.authenticate.assert_called_once()
         self.assertEqual(response.json, third_response.json)
 
+    def test_timed_out_session_fail_after_reestablish(self):
+        self.auth._session_key = 'asdf1234'
+        self.auth.get_session_key.return_value = 'asdf1234'
+        self.conn._auth = self.auth
+        self.session = mock.Mock(spec=requests.Session)
+        self.session.auth = None
+        self.conn._session = self.session
+        self.request = self.session.request
+        first_response = mock.MagicMock()
+        first_response.status_code = http_client.FORBIDDEN
+        second_response = mock.MagicMock()
+        second_response.status_code = http_client.BAD_REQUEST
+        self.request.side_effect = [first_response, second_response]
+        self.assertRaises(exceptions.BadRequestError,
+                          self.conn._op,
+                          'POST', path='fake/path', data=self.data,
+                          headers=self.headers)
+        self.auth.refresh_session.assert_called_with()
+        self.auth.can_refresh_session.assert_called_with()
+
+    def test_timed_out_session_fail_after_reestablish_no_recursion(self):
+        self.auth._session_key = 'asdf1234'
+        self.auth.get_session_key.return_value = 'asdf1234'
+        self.conn._auth = self.auth
+        self.session = mock.Mock(spec=requests.Session)
+        self.session.auth = None
+        self.conn._session = self.session
+        self.request = self.session.request
+        first_response = mock.MagicMock()
+        first_response.status_code = http_client.FORBIDDEN
+        second_response = mock.MagicMock()
+        second_response.status_code = http_client.FORBIDDEN
+        self.request.side_effect = [first_response, second_response]
+        self.assertRaises(exceptions.AccessError,
+                          self.conn._op,
+                          'POST', path='fake/path', data=self.data,
+                          headers=self.headers)
+        self.auth.refresh_session.assert_called_with()
+        self.auth.can_refresh_session.assert_called_with()
+
     def test_connection_error(self):
         self.request.side_effect = requests.exceptions.ConnectionError
         self.assertRaises(exceptions.ConnectionError, self.conn._op, 'GET')
@@ -482,6 +545,29 @@ class ConnectorOpTestCase(base.TestCase):
         self.assertEqual(0, mock_sleep.call_count)
         self.assertEqual(1, self.request.call_count)
 
+    def test_op_retry_without_identity(self):
+        self.request.side_effect = [
+            mock.Mock(status_code=http_client.NOT_ACCEPTABLE),
+            mock.Mock(status_code=http_client.OK),
+        ]
+        self.conn._op('GET', 'http://foo.bar')
+
+        self.assertEqual(2, self.request.call_count)
+        headers_no_accept = self.headers.copy()
+        headers_no_accept.pop('Accept-Encoding')
+        self.request.assert_has_calls([
+            mock.call('GET', 'http://foo.bar', headers=self.headers,
+                      json=None, verify=True, timeout=60),
+            mock.call('GET', 'http://foo.bar', headers=headers_no_accept,
+                      json=None, verify=True, timeout=60),
+        ])
+
+    def test_op_retry_without_identity_fails(self):
+        self.request.return_value.status_code = http_client.NOT_ACCEPTABLE
+        self.assertRaises(exceptions.NotAcceptableError, self.conn._op,
+                          'GET', 'http://foo.bar')
+        self.assertEqual(2, self.request.call_count)
+
     def test_access_error(self):
         self.conn._auth = None
 
@@ -675,8 +761,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header',
-                     'If-Match': '"3d7b8a7360bf2941d"'},
+            headers=dict(self.headers, **{'If-Match': '"3d7b8a7360bf2941d"'}),
             blocking=False,
             timeout=60)
 
@@ -698,8 +783,8 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header',
-                     'If-Match': 'W/"3d7b8a7360bf2941d"'},
+            headers=dict(self.headers,
+                         **{'If-Match': 'W/"3d7b8a7360bf2941d"'}),
             blocking=False,
             timeout=60)
 
@@ -719,7 +804,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header'},
+            headers=self.headers,
             blocking=False,
             timeout=60)
 
@@ -747,8 +832,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header',
-                     'If-Match': '"3d7b8a7360bf2941d"'},
+            headers=dict(self.headers, **{'If-Match': '"3d7b8a7360bf2941d"'}),
             blocking=False,
             timeout=60)
 
@@ -778,7 +862,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header'},
+            headers=self.headers,
             blocking=False,
             timeout=60)
 
@@ -806,7 +890,7 @@ class ConnectorOpTestCase(base.TestCase):
             '/redfish/v1/Systems/1',
             data={'Boot': {'BootSourceOverrideTarget': 'Cd',
                            'BootSourceOverrideEnabled': 'Once'}},
-            headers={'X-Fake': 'header'},
+            headers=self.headers,
             blocking=False,
             timeout=60)
 
